
fn main() {
    env_logger::init_from_env(
        env_logger::Env::default().filter_or(env_logger::DEFAULT_FILTER_ENV, "info"));

    let args: Vec<String> = std::env::args().collect();
    if args.len() != 2 {
        log::error!("Usage: {} foo.gmf", args[0]);
        return;
    }
    let gmf_path = &args[1];

    let f = match std::fs::File::open(gmf_path) {
        Ok(f) => f,
        Err(e) => {
            log::error!("Could not open {}: {:?}", gmf_path, e);
            return;
        },
    };

    let gmf = match gmflib::GMF::read_gmi(f) {
        Ok(gmf) => gmf,
        Err(e) => {
            log::error!("Reading GMI {} failed: {}", gmf_path, e.msg);
            log::info!("Parse stack:");
            for frame in e.stack.iter() {
                log::info!("   {} at offset {}", frame.name, frame.loc);
            }
            return;
        },
    };

    let mut buf: Vec<u8> = vec![];
    if let Err(e) = gmf.write_gma(&mut buf) {
        log::error!("Writing GMA failed: {}", e.msg);
    }
}
